@@ -1,11 +1,14 @@
 #!/usr/bin/env python
 import inspect
-from objectname import objectname
-from public import public
+import public
+
+"""
+f.func_code.co_argcount  python 2.x only
+"""
 
 
-def err_msg(func, args, pos, required_types):
-    name = objectname(func, fullname=True)
+def _err_msg(func, args, pos, required_types):
+    name = func.__name__
     lines = [
         "%s() argument #%s is not instance of %s" %
         (name, pos, required_types)]
@@ -13,15 +16,18 @@ def err_msg(func, args, pos, required_types):
     lines += ["arg#%s: %s" % (pos, args[pos])]
     return "\n".join(lines)
 
-# f.func_code.co_argcount  python 2.x only
+
+def _func_args(f):
+    try:
+        # getargspec() is deprecated in python3, so use drop-in-replacement getfullargspec() whenever available
+        return len(inspect.getfullargspec(f).args)
+    except AttributeError:
+        # fall back to getargspec() if getfullargspec() is not available (python2)
+        return len(inspect.getargspec(f).args)
 
 
-def func_args(f):
-    return len(inspect.getargspec(f).args)
-
-
-def validate_args(f, args, types):
-    args_count = func_args(f)
+def _validate_args(f, args, types):
+    args_count = _func_args(f)
     if args_count != len(types):  # python 2.x
         msg = "%s arguments. expected: %s" % (len(types), args_count)
         raise ValueError(msg)
@@ -29,15 +35,16 @@ def validate_args(f, args, types):
         # isinstance() arg 2 must be a type or tuple of types
         # fix: type(None)
         if not isinstance(arg, required_types):
-            msg = err_msg(f, args, pos, required_types)
+            msg = _err_msg(f, args, pos, required_types)
             raise TypeError(msg)
 
 
-@public
+@public.add
 def accepts(*types):
+    """@accepts decorator"""
     def check_accepts(f):
         def new_f(*args, **kwargs):
-            validate_args(f, args, types)
+            _validate_args(f, args, types)
             return f(*args, **kwargs)
         try:  # python2
             new_f.func_name = f.func_name
